/*
   Initialization functions
   --------------------------------------------------------------------
   VCHE - Virtual Console Hex Editor

   Copyright (C) 1998 Diego Javier Grigna <diego@grigna.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"

/*
 * Function prototypes
 */
static void init_read_vcss( void);
static void init_check_config_file( void);
static void init_process_config( char *buf);
static int init_get_token( void);

extern char *hbuffer;
extern char *hex_buffer;

int vcsafd;              /* File descriptor of the /dev/vcsa* file   */
int vcsfd;               /* File descriptor of the /dev/vcs* file    */
char **vcs_save_buffer;  /* Buffer for saving and restoring screens  */
int vcs_save_index;      /* Index for the save/restore screen buffer */

static char *token;
static const char delimiters[] = "=\t\n";

void init( void)
{
 fpos = 0;

 signals_init();

 term_init();

 init_read_vcss();

 term_hide_cursor();

 init_buffers();

 flag_bell     = 0;
 flag_edit     = 0;
 flag_mask     = 0;
 flag_nohex    = 0;
 flag_readonly = 0;
 flag_zero     = 0;

 init_check_config_file();

 if( flag_readonly && flag_edit) {
     fprintf( stderr, "\nConfig file error: Flags \"readonly\" and \"edit\" cannot be set to \"on\"\nat the same time.\n\n");
     do_exit( 1);
 }

 vcs_save_buffer = ( char **) allocate_mem( MAX_VCS_SAVE * sizeof( char *));

 draw_main_screen();

 set_flag( FLAG_BELL);
 set_flag( FLAG_EDIT);
 set_flag( FLAG_MASK);
 set_flag( FLAG_NOHEX);
 set_flag( FLAG_READONLY);
 set_flag( FLAG_ZERO);

 vcs_save_index = 0;

 edit_mask = 0;

 bzero( &srch, 63);

 set_hexmode_variables();
 
 vc_fill_obuffer();
}

static void init_read_vcss( void)
{
 char vcsadevname[ 256];
 char vcsdevname[ 256];
 char *mytty;
 int vcnum;

 if( (mytty = ttyname( fileno( stdin))) == NULL) {
     perror( "stdin"); 
     exit( 1);
 }
 
 if( sscanf( mytty, "/dev/tty%d", &vcnum) != 1) { 
     fprintf( stderr, "This program only runs in a system console!\n");
     fprintf( stderr, "Run vche-nc or vche-raw instead.\n");
     exit(1);
 }

 sprintf( vcsadevname, "/dev/vcsa%d", vcnum);

 if( (vcsafd = open( vcsadevname, O_RDWR)) == -1) {
     fprintf( stderr, "Can't open \"%s\" : %s\n", strerror( errno), vcsadevname);
     exit( 1);
 }

 read( vcsafd, &scrn, 4);

 sprintf( vcsdevname, "/dev/vcs%d", vcnum);

 if( (vcsfd = open( vcsdevname, O_RDWR)) == -1) {
     fprintf( stderr, "Can't open \"%s\" : %s\n", strerror( errno), vcsdevname);
     exit( 1);
 }

}

static void init_check_config_file( void)
{
 char config_file[ 2048];
 char buf[ 8192];
 FILE *fp;

#ifdef VCHE_LIBDIR
 sprintf( config_file, "%s/vcherc", VCHE_LIBDIR);
#else
 sprintf( config_file, "/usr/local/lib/vche/vcherc");
#endif

 /* If the config file does not exist return silently */
 if( access( config_file, F_OK) == -1) return;

 if( (fp = fopen( config_file, "r")) == NULL) {
     fprintf( stderr, "Can't open \"%s\" : %s\n", config_file, strerror( errno));
     do_exit( 1);
 }

 while( !feof( fp) && fgets( buf, sizeof( buf) - 1, fp) != NULL) {
        if( buf[ 0] == '#' || buf[ 0] == '\n') continue;
        init_process_config( buf);
 }

 fclose( fp);
}

static void init_process_config( char *buf)
{
 token = strtok( buf, delimiters);

 if( token == NULL) return;

 if( !strcasecmp( token, "bell"    )) flag_bell     = init_get_token();
 if( !strcasecmp( token, "edit"    )) flag_edit     = init_get_token();
 if( !strcasecmp( token, "mask"    )) flag_mask     = init_get_token();
 if( !strcasecmp( token, "nohex"   )) flag_nohex    = init_get_token();
 if( !strcasecmp( token, "readonly")) flag_readonly = init_get_token();
 if( !strcasecmp( token, "zero"    )) flag_zero     = init_get_token();
}

static int init_get_token( void)
{
 token = strtok( NULL, delimiters);

 if( token == NULL) return 0;

 if( !strcasecmp( token,  "on")) return 1;

 if( !strcasecmp( token, "off")) return 0;

 return 0;
}

void init_buffers( void)
{
 hex_buffer = ( char *) allocate_mem( scrn.cols);
 hbuffer    = ( char *) allocate_mem( ( scrn.lines - 2 ) * scrn.cols);
 obuffer    = ( char *) allocate_mem( scrn.cols * 2 * ( scrn.lines - 2));
}

